%% SCRIPT FOR ANALYSIS ON THE CREATED DATASETS

% Julia Gala de Pablo
% PhD Student
% University of Leeds

%% FILE IMPORTING
clear all
close all
clc

dirPath='C:\SavingPathExample';
addpath('C:\MatlabFunctionsPath'); % Containing the function plotspread
NameSaving{1}='Celltype_group_1_repeat_1';
NameSaving{2}='Celltype_group_1_repeat_2';
NameSaving{3}='Celltype_group_2_repeat_1';
NameSaving{4}='Celltype_group_2_repeat_2';

% Name of the particular experiment for the dataset imported
NameLegend={...
    'Cell type and group 1 repeat 1';...
	'Cell type and group 1 repeat 2';...
    'Cell type and group 2 repeat 1';...
    'Cell type and group 2 repeat 2';...    
   };

% Multiple experiments can constitute the same class, name of Class:
NameLegendClass={
    'Cell type and group 1';...
    'Cell type and group 2';...
    };

NameRaw='_datasetraw';
NameFinalData='_datasetanalysed';

selectedDatasets=[1:4];

fprintf('Loading the Data \n')

% Starting variables
FileName={}; m=1; RamanSpectra={}; RamanBackgroundSpectra={};
DataMatrix=[]; Average={}; Error={};

for j=selectedDatasets
    fprintf(['Loading data from dataset' num2str(j) '\n']);
    cd(dirPath);
    % Loading Raw Data
    rawTemp=load([NameSaving{j} NameRaw '.mat']);
    NumOfCells(j)=length(rawTemp.RamanSpectra);
    disp(['Number of cells: ' num2str(NumOfCells(j))])
    Index{j}=[length(RamanSpectra)+1:length(RamanSpectra)+NumOfCells(j)];
    RamanSpectra=[RamanSpectra rawTemp.RamanSpectra];
    XvaluesTrunc2=rawTemp.XvaluesTrunc2;
    CalibrationPeak{j}=rawTemp.CalibrationPeakTemp;
    RamanBackgroundSpectra=[RamanBackgroundSpectra rawTemp.RamanBackgroundSpectra];
    
    % Loading Data Matrix
    analTemp=load([NameSaving{j} NameFinalData '.mat']);
    DataMatrix=[DataMatrix analTemp.DataMatrix];
    Average{j}=analTemp.Average;
    Error{j}=analTemp.Error; % just the std, not the standar error
    clear rawTemp analTemp
end


%% Create the classes for each cell line

Class{1}=[Index{1} Index{2}];
Class{2}=[Index{3} Index{4}];

% Colors for plots
colors={'b','c',...
    'r','m'};
colorsClass={'b','r'};

%% Plotting all the raw spectra
fprintf('Plotting the raw spectra \n')

OffsetPlot=0;

figure(1)
for j=selectedDatasets
    hold on
    for k=Index{j}
        subplot(1,2,1)
        hold on
        RawPlot{k}=plot(RamanSpectra{k}(:,1),RamanSpectra{k}(:,2)+OffsetPlot*(j-1),colors{j});
        subplot(1,2,2)
        hold on
        plot(RamanSpectra{k}(:,1),RamanSpectra{k}(:,2)+OffsetPlot*(j-1),colors{j});  
    end
    LegendPlot{j}=RawPlot{end};
    clear RawPlot
end

subplot(1,2,1,'align')
xlim([500 1800])
box on

subplot(1,2,2,'align')
xlim([2800 3050])
box on
xlabel('Wavenumber (cm^{-1})')

LegendString={};
for j=selectedDatasets
    LegendString=[LegendString [NameLegend{j} '(N=' num2str(NumOfCells(j)) ')']];
end

LegendStringClass={};
for j=1:length(Class)
    LegendStringClass=[LegendStringClass [NameLegendClass{j} '(N=' num2str(length(Class{j})) ')']];
end

subplot(1,2,1)
legend([LegendPlot{selectedDatasets}],...
    LegendString)
xlabel('Shift (cm^{-1})')
ylabel('Intensity (Offset)')
title('Raw spectra')


%% Plotting all the spectra after editting
fprintf('Plotting the editted spectra \n')

OffsetPlot=1.8;
figure(2)

cd(dirPath)

for j=selectedDatasets
    hold on
    for k=Index{j}
        subplot('Position',[0.1,0.1,0.55,0.8])
        hold on
        RawPlot{k}=plot(XvaluesTrunc2,DataMatrix(:,k)+OffsetPlot*(j-1),colors{j});
        subplot('Position',[0.7,0.1,0.2,0.8])
        hold on
        plot(XvaluesTrunc2,DataMatrix(:,k)+OffsetPlot*(j-1),colors{j});  
    end
    LegendPlot{j}=RawPlot{end};
    clear RawPlot
end

subplot('Position',[0.1,0.1,0.55,0.8])
xlim([min(XvaluesTrunc2) 1800])
box on

subplot('Position',[0.7,0.1,0.2,0.8])
xlim([2800 max(XvaluesTrunc2)])
box on
xlabel('Shift (cm^{-1})')

subplot('Position',[0.1,0.1,0.55,0.8])
legend([LegendPlot{selectedDatasets}],...
    LegendString)
xlabel('Shift (cm^{-1})')
ylabel('Intensity (Offset)')
title('Edited spectra')


%% Plotting all the edited spectra by class
fprintf('Plotting the editted spectra \n')

OffsetPlot=2;
figure(3)

for j=1:length(Class)
    hold on
    for k=Class{j}
        subplot('Position',[0.1,0.1,0.58,0.85])
        hold on
        RawPlot{k}=plot(XvaluesTrunc2,DataMatrix(:,k)+OffsetPlot*(j-1),colorsClass{j});
        subplot('Position',[0.7,0.1,0.25,0.85])
        hold on
        plot(XvaluesTrunc2,DataMatrix(:,k)+3*OffsetPlot*(j-1),colorsClass{j});  
    end
    LegendPlot{j}=RawPlot{end};
    clear RawPlot
end

subplot('Position',[0.1,0.1,0.58,0.85])
xlim([min(XvaluesTrunc2) 1750])
ylim([-OffsetPlot/3 OffsetPlot*(length(Class)+0.8)])
legend([LegendPlot{1:length(Class)}],...
    LegendStringClass,'box','off')
xlabel('Wavenumber / cm^{-1}')
ylabel('Raman Intensity / normalized offset')
box on

subplot('Position',[0.7,0.1,0.25,0.85])
xlim([2800 max(XvaluesTrunc2)])
ylim([-OffsetPlot/3 OffsetPlot*3*(length(Class)+0.8)])
text(3000,OffsetPlot*3*(length(Class)),'x1/3')
box on
xlabel('Wavenumber / cm^{-1})')
set(gca,'YTickLabels',{})

clear LegendPlot


%% Plot the averages of each experiment
fprintf('Plotting the average spectra \n')


OffsetPlot=0;%1.2;
figure(4)
subplot(1,2,1)
xlabel('Shift (cm^{-1})')
ylabel('Normalized Intensity (Offset)')
hold on
for j=selectedDatasets
    subplot('Position',[0.1,0.1,0.55,0.8])
    hold on
    AveragePlot{j}=plot(XvaluesTrunc2,Average{j}+OffsetPlot*(j-1),colors{j});
    xlim([min(XvaluesTrunc2) 1800])
    subplot('Position',[0.7,0.1,0.2,0.8])
    hold on
    plot(XvaluesTrunc2,Average{j}+OffsetPlot*3*(j-1),colors{j});
    xlim([2800 max(XvaluesTrunc2)])
end    

subplot('Position',[0.1,0.1,0.55,0.8])
box on
xlabel('Shift (cm^{-1})')
ylabel('Normalized Intensity')
title('Spectrum Average')
legend([AveragePlot{selectedDatasets}],...
    LegendStringClass)
subplot('Position',[0.7,0.1,0.2,0.8])
box on
xlabel('Shift (cm^{-1})')
ylabel('Normalized Intensity')



%% Claculate the Class averages after editting and plot the difference to the average

for j=1:length(Class)
    AverageClass{j}=mean(DataMatrix(:,Class{j}),2);
    ErrorClass{j}=std(DataMatrix(:,Class{j}),[],2);
end

AverageAll=mean(DataMatrix,2);

fprintf('Plotting the average spectra \n')


OffsetPlot=0;
figure(5)
subplot(1,2,1)
xlabel('Shift (cm^{-1})')
ylabel('Normalized Intensity (Offset)')
hold on
for j=1:length(Class)
    subplot('Position',[0.1,0.1,0.55,0.8])
    hold on
    AveragePlotClass{j}=plot(XvaluesTrunc2,sgolayfilt(AverageClass{j}-AverageAll,2,15)+OffsetPlot*(j-1),colorsClass{j});
    xlim([min(XvaluesTrunc2) 1750])
    subplot('Position',[0.7,0.1,0.2,0.8])
    hold on
    plot(XvaluesTrunc2,sgolayfilt(AverageClass{j}-AverageAll,2,15)+OffsetPlot*3*(j-1),colorsClass{j});
    xlim([2800 max(XvaluesTrunc2)])
end    


subplot('Position',[0.1,0.1,0.55,0.8])
line([min(XvaluesTrunc2) 1750],[0 0],'Color','k')
box on
xlabel('Shift (cm^{-1})')
ylabel('Normalized Intensity')
title('Spectrum Average')
legend([AveragePlotClass{1:length(Class)}],...
    LegendStringClass)
subplot('Position',[0.7,0.1,0.2,0.8])
line([2800 max(XvaluesTrunc2)],[0 0],'Color','k')
box on
xlabel('Shift (cm^{-1})')
ylabel('Normalized Intensity')



%% Discriminant Analysis

% To use fitcdiscr it it needs a matrix of predictors (that would be the
% matrix with the scores of PCA) and an array with the responses (the
% clases being introduced, such as the cell type and the conditions of the
% experiment). We can build a responses array from the index:
fprintf('Creating the Responses array for Discriminant Analysis\n')

ClassesTitles={'Cell type and group 1','Cell type and group 2'};

Responses={};
ResponsesNumeric=[];
for j=1:length(Class)
    for i=Class{j}
        Responses(end+1)=strcat(ClassesTitles(j));
        ResponsesNumeric=[ResponsesNumeric; j];
    end
end


%% Perform the tree fit
disp('Testing the final model choices')
NumberOfTests=5;
kfold=10;
count=1;
for k=1:NumberOfTests
    % Chooses randomly N cells from each cell's selected indexes:
    
    % TRAINING
    % The function crossvalind labels each cell into one of the k-folds for
    % then implementing the k-fold cross validation.
    Indices=crossvalind('Kfold',size(DataMatrix,2),kfold);
    cp1 = classperf(Responses); % initializing the performance object 
    for i=1:kfold %over the k folds
        disp(['Running for fold ' num2str(i) ' out of ' num2str(NumberOfTests*10)])
        % logical vector that is only 1 for the current testing k-fold
        TestVector=(Indices==i);
        % rest of the indices are used for training
        TrainVector=~TestVector;
        % TRAINING THE MODEL with the Training Vector
        DecisionTreeRamanF = fitctree (DataMatrix(:,TrainVector).',...
            Responses(TrainVector),'MaxNumSplits',50);...
            %'AlgorithmForCategorical','Exact');%,...
            %'PredictorNames',PredictorNames);
        % TESTING with the remaining cells
        ypF{count}=predict(DecisionTreeRamanF,...
            DataMatrix(:,TestVector).');
        cpF{count}=classperf(ypF{i},Responses(TestVector));
        Sensitivity(count)=cpF{count}.Sensitivity;
        count=count+1;
    end
    clear Indices
end

%% Final performance
SensitivityF=mean(Sensitivity);
SensitivityFSE=std(Sensitivity)/sqrt(NumberOfTests*10); %testsxFolds

disp(['Sensitivity 5 rep kfold 10 Cross val Tr 64 of ',...
    num2str(SensitivityF*100),'+-',...
    num2str(SensitivityFSE*100),'%'])

% Visualize one of the trees from one of the k-folds
view(DecisionTreeRamanF,'Mode','graph')

clear Sensitivity cpF ypF TrainVector TestVector Indices

%% 3D plot of the spectra - Final tree results
% Scan the best shifts in order to make the decision

disp('Obtaining the main shift bands from the discrimination tree')

ShiftF=[];
for i=1:length(DecisionTreeRamanF.CutPredictor)
    ShiftTemp=sscanf(DecisionTreeRamanF.CutPredictor{i},'x%f',1);
    if not(isempty(ShiftTemp))
        ShiftF=[ShiftF ShiftTemp];
    end
end

% Create a vector with the chosen shifts from this example tree for
ShiftFexport=ShiftF;
clear Shift1 Shift2 Shift3

Shift1=ShiftF(1);
Shift2=ShiftF(2);
Shift3=ShiftF(3);

% 3D plot the data

fprintf('3D-Plotting the cell Shifts \n')
figure

hold on
for j=1:length(Class)
    hold on
    scatter3(DataMatrix(Shift1,Class{j}),...
        DataMatrix(Shift2,Class{j}),...
        DataMatrix(Shift3,Class{j}),...
        colorsClass{j},'o','filled');
end

xlabel(['Normalized I_{',num2str(XvaluesTrunc2(Shift1)),'cm^{-1}}'])
ylabel(['Normalized I_{',num2str(XvaluesTrunc2(Shift2)),'cm^{-1}}'])
zlabel(['Normalized I_{',num2str(XvaluesTrunc2(Shift3)),'cm^{-1}}'])

grid on
box on
axis square tight
view(3)
legend(LegendStringClass)

xlim manual
ylim manual
zlim manual



%% Plotting all the spectra (per class) after editting marking the relevant peaks
fprintf('Plotting the editted spectra \n')

OffsetPlot=1.2;
figure(8)
subplot(1,2,1)
xlabel('Shift (cm^{-1})')
ylabel('Normalized Intensity')
hold on
count=0;
for j=1:length(Class)
    subplot('Position',[0.1,0.1,0.55,0.8])
    hold on
    AveragePlot{j}=plot(XvaluesTrunc2,AverageClass{j}+OffsetPlot*count,colorsClass{j});
    xlim([min(XvaluesTrunc2) 1750])
    subplot('Position',[0.7,0.1,0.2,0.8])
    hold on
    plot(XvaluesTrunc2,AverageClass{j}+OffsetPlot*3*count,colorsClass{j});
    xlim([2800 max(XvaluesTrunc2)])
    count=count+1;
end    

subplot('Position',[0.1,0.1,0.55,0.8])
box on
xlabel('Shift (cm^{-1})')
ylabel('Normalized Intensity')
ylim([0 OffsetPlot*(count+0.5)])
title('Spectrum Average')
legend([AveragePlot{1:length(Class)}],...
    LegendStringClass)
subplot('Position',[0.7,0.1,0.2,0.8])
box on
ylim([0 OffsetPlot*(count+0.5)*3])
xlabel('Shift (cm^{-1})')
ylabel('Normalized Intensity')

for i=1:length(ShiftF)
    subplot('Position',[0.1,0.1,0.55,0.8])
    line([XvaluesTrunc2(ShiftF(i)) XvaluesTrunc2(ShiftF(i))], [0 OffsetPlot*13]);
    subplot('Position',[0.7,0.1,0.2,0.8])
    line([XvaluesTrunc2(ShiftF(i)) XvaluesTrunc2(ShiftF(i))], [0 OffsetPlot*45]);    
end

clear AveragePlot

%% Standarizing the data

% SNV (Standar Normal Variate) normalizes the standar deviation of each 
% spectra to 1. It is calculated as:

% SNV = (Data - Average)/Standar deviation

% Calculating the average and stdev of the columns (each column is a spectra)
fprintf('Standarizing the PCA Data Matrix \n')
RamanSpectraAverage=mean(DataMatrix,1);
RamanSpectraSTD=std(DataMatrix,0,1);

for i=1:size(DataMatrix,2)
    for j=1:size(DataMatrix,1)
        DataMatrixNorm(j,i)=(DataMatrix(j,i)-RamanSpectraAverage(i))./RamanSpectraSTD(i);
    end
end

% Each cell needs to be a different row, obtaining the coefficients as:
[coeff,score,latent,tsquared,explained,mu]=pca((DataMatrixNorm.'));

%% Plot variance explained

figure(100)
bar(explained,0.9,'k')
xlim([0.5 10.5])
xlabel('Components')
ylabel('Variance explained (%)')
hold on
plot(cumsum(explained),'sk-')
legend('Variance','Cumulative variance')


%% Plot the components
XvaluesTrunc2PCA=XvaluesTrunc2(1:2444);

fprintf('Plotting the main 3 first components \n')
score1=1; score2=2; score3=3; score4=4;

figure(5)
offsetValue=0.12;
hold on

subplot('Position',[0.1,0.1,0.55,0.8])
hold on
plot(XvaluesTrunc2PCA,coeff(:,score1)+offsetValue*3,'g',...
    XvaluesTrunc2PCA,coeff(:,score2)+offsetValue*2,'b',...
    XvaluesTrunc2PCA,coeff(:,score3)+offsetValue,'k',...
    XvaluesTrunc2PCA,coeff(:,score4),'r')
line([XvaluesTrunc2(1) XvaluesTrunc2(end)], [offsetValue offsetValue])
line([XvaluesTrunc2(1) XvaluesTrunc2(end)], [offsetValue*2 offsetValue*2])
line([XvaluesTrunc2(1) XvaluesTrunc2(end)], [offsetValue*3 offsetValue*3])
line([XvaluesTrunc2(1) XvaluesTrunc2(end)], [0 0])
subplot('Position',[0.7,0.1,0.2,0.8])
hold on
plot(XvaluesTrunc2PCA,coeff(:,score1)+offsetValue*3,'g',...
    XvaluesTrunc2PCA,coeff(:,score2)+offsetValue*2,'b',...
    XvaluesTrunc2PCA,coeff(:,score3)+offsetValue,'k',...
    XvaluesTrunc2PCA,coeff(:,score4),'r') 
line([XvaluesTrunc2(1) XvaluesTrunc2(end)], [offsetValue offsetValue])
line([XvaluesTrunc2(1) XvaluesTrunc2(end)], [offsetValue*2 offsetValue*2])
line([XvaluesTrunc2(1) XvaluesTrunc2(end)], [offsetValue*3 offsetValue*3])
line([XvaluesTrunc2(1) XvaluesTrunc2(end)], [0 0])
subplot('Position',[0.1,0.1,0.55,0.8])
xlim([min(XvaluesTrunc2) 1800])
box on

subplot('Position',[0.7,0.1,0.2,0.8])
xlim([2800 max(XvaluesTrunc2)])
box on
xlabel('Shift (cm^{-1})')

subplot('Position',[0.1,0.1,0.55,0.8])
xlabel('Shift (cm^{-1})')
ylabel('Intensity (Offset)')
title(['Components ' num2str(score1) ', ' num2str(score2) ' and ' ...
    num2str(score3)])
legend(['PC' num2str(score1)],['PC' num2str(score2)],...
    ['PC' num2str(score3)],['PC' num2str(score4)])

%% Plot the scores
%Scores to plot
fprintf('Plotting the cell scores \n')

figure(6)

hold on
for j=1:length(Class)
    scatter3(score(Class{j},score1),...
        score(Class{j},score2),...
        score(Class{j},score3),...
        50,colorsClass{j},...
        'o','filled');
    hold on
end

xlabel(['score of PC',num2str(score1)])
ylabel(['score of PC',num2str(score2)])
zlabel(['score of PC',num2str(score3)])
grid on
box on
axis square tight
view(3)
legend(LegendStringClass)


%% Plot hist of the scores

figure(105)
subplot(1,3,1)
hold on
count=1;
for k=1:8
    subplot(1,4,count)
    for j=1:length(Class)
        histogram(score(Class{j},k),15,'FaceColor',colorsClass{j},'BinWidth',1);
        hold on
    end
    box on
    xlabel('Score')
    ylabel('Number of cells')
    title(['PC' num2str(k)])
    count=count+1;
end

legend(LegendStringClass)

%% Plot the box plots of the scores

figure
hold on
for k=1:8
    subplot(2,4,k)
    boxplot(score(:,k),Responses)
    plotSpread(score(:,k),'distributionIdx',ResponsesNumeric,...
        'categoryLabels',LegendStringClass,...
        'distributionColors',{'g','m','k','b','r'})    
    axis square
    title(strcat('PC',num2str(k)))
end

%% LDA model
disp('Testing the final model choices')
ComponentsToKeep=find(cumsum(explained)>95,1);
NumberOfTests=5;
kfold=10;
count=1;
for i=1:NumberOfTests
    % Chooses randomly N cells from each cell's selected indexes:
    % TRAINING
    % The function crossvalind labels each cell into one of the k-folds for
    % then implementing the k-fold cross validation.
    Indices=crossvalind('Kfold',size(DataMatrix,2),kfold);
    cp1 = classperf(Responses); % initializing the performance object 
    for j=1:kfold %over the k folds
        disp(['Running for fold ' num2str(count) ' out of ' num2str(NumberOfTests*10)])
        % logical vector that is only 1 for the current testing k-fold
        TestVector=(Indices==j);
        % rest of the indices are used for training
        TrainVector=~TestVector;
        % TRAINING THE MODEL with the Training Vector
        LDAobjectF = fitcdiscr(score(TrainVector,1:ComponentsToKeep),...
            Responses(TrainVector));
        % TESTING with the remaining cells
        ypLDAF{j}=predict(LDAobjectF,...
            score(TestVector,1:ComponentsToKeep));
        cpLDAF{count}=classperf(ypLDAF{j},Responses(TestVector));
        CorrectRate(count)=cpLDAF{count}.CorrectRate;
        Sensitivity(count)=cpLDAF{count}.Sensitivity;
        Specificity(count)=cpLDAF{count}.Specificity;
        [C,c]=confusionmat(Responses(TestVector).',ypLDAF{j});
        ConfMat(:,:,count)=C;
        count=count+1;
    end
end

% Final performance
LDACorrectRateF=mean(CorrectRate);
LDACorrectRateFSE=std(CorrectRate)/sqrt(NumberOfTests*10); %testsxFolds
LDASensitivityF=mean(Sensitivity);
LDASensitivityFSE=std(Sensitivity)/sqrt(NumberOfTests*10);
LDASpecificityF=mean(Specificity);
LDASpecificityFSE=std(Specificity)/sqrt(NumberOfTests*10);

disp(['Sensitivity 5 rep kfold 10 Cross val of ',...
    num2str(LDASensitivityF*100),'+-',...
    num2str(LDASensitivityFSE*100),'%'])
disp(['Specificity 5 rep kfold 10 Cross val of ',...
    num2str(LDASpecificityF*100),'+-',...
    num2str(LDASpecificityFSE*100),'%'])
disp(['Correct rate 5 rep kfold 10 Cross val of ',...
    num2str(LDACorrectRateF*100),'+-',...
    num2str(LDACorrectRateFSE*100),'%'])

% Calculating the confusion matrix values
ConfMatPerc=100.*ConfMat./repmat(sum(ConfMat,1),[size(ConfMat,1),1]);
ConfMatPercAv=mean(ConfMatPerc,3);
ConfMatPercErr=std(ConfMatPerc,0,3)./sqrt(50);

%% Coefficients:
% Obtaining the coefficients from one of the trained models (the last one):
count=1;
clear LDAF
for i=1:length(Class)
    for j=1:length(Class)
        if and(i~=j,i>j)
            LDAF.Class(:,count)=[ones(size(score,1),1) score(:,1:ComponentsToKeep)] * ...
                [LDAobjectF.Coeffs(j,i).Const LDAobjectF.Coeffs(j,i).Linear.'].';
            LDAF.Coeff(:,count)=[LDAobjectF.Coeffs(j,i).Linear.']*...
                [coeff(:,1:ComponentsToKeep)].';
            LDAF.Class1(count)=i;
            LDAF.Class2(count)=j;
            count=count+1;
        end
    end
end

%% Plot the results as histograms and scatter plots
figure(221)
count=1;
for i=1:size(LDAF.Class,2)
    subplot(3,4,count)
    title(strcat(LDAobjectF.ClassNames(LDAF.Class1(i)),'vs',...
        LDAobjectF.ClassNames(LDAF.Class2(i))))
        for j=[LDAF.Class1(i) LDAF.Class2(i)]
            hold on
            histogram(LDAF.Class(Class{j},i),'FaceColor',colorsClass{j})
        end
    line([0 0],[0 50],'Color','k','LineWidth',1.5)
    count=count+1;
    xlabel(['LDA score' num2str(i)])
    ylabel('Number of cells')
end

selectedLD=[1:6];

for k=1:2:selectedLD(end)
    subplot(3,3,count)
    for j=1:length(Class)
        hold on
        scatter(LDAF.Class(Class{j},selectedLD(k)),LDAF.Class(Class{j},...
            selectedLD(k+1)),'o','FaceColor',colorsClass{j})
    end
    xlabel(['LDA score ' num2str(selectedLD(k))])
    ylabel(['LDA score ' num2str(selectedLD(k+1))])    
    count=count+1;
end

legend(LegendStringClass)

%% Plot the box plots and beeswarms of the scores

figure
hold on
for k=1:10
    subplot(2,5,k)
    hold on
    boxplot(LDAF.Class(:,k),Responses)
    plotSpread(LDAF.Class(:,k),'distributionIdx',ResponsesNumeric,...
        'categoryLabels',LegendStringClass,...
        'distributionColors',{'g','m','k','r','b'})
    line([0 length(LegendStringClass)+1],[0 0],'Color','k','LineWidth',1.5)
    axis square
    title(strcat('LD',num2str(k),...
        '(',...
        LDAobjectF.ClassNames(LDAF.Class1(k)),'vs',...
        LDAobjectF.ClassNames(LDAF.Class2(k)),...
        ')'))
    ylabel('Score')
end

%% Plotting all the LDs shapes
OffsetPlot=2;
figure(3)
colormap('hsv')
c=colormap;

selectedCoeff=1:size(LDAF.Coeff,2);

count=0;
for j=selectedCoeff
    subplot('Position',[0.1,0.1,0.55,0.8])
    hold on
    plot(XvaluesTrunc2PCA,LDAF.Coeff(:,j)+OffsetPlot*count,'Color',c(j*6,:));
    subplot('Position',[0.7,0.1,0.2,0.8])
    hold on
    plot(XvaluesTrunc2PCA,LDAF.Coeff(:,j)+OffsetPlot*count,'Color',c(j*6,:));  
    count=count+1;
end

subplot('Position',[0.1,0.1,0.55,0.8])
xlim([min(XvaluesTrunc2) 1750])
xlabel('Shift (cm^{-1})')
ylabel('Normalized Intensity (Offset)')
box on

subplot('Position',[0.7,0.1,0.2,0.8])
xlim([2800 max(XvaluesTrunc2)])
box on
xlabel('Shift (cm^{-1})')

clear LegendPlot

%% Calculating the correlation matrices and turning that into images
% Of the DataMatrix
% To use the function corrcoef the columns are variables (shift values) and
% the rows are observations (cell numbers), so we need to use the
% transposed matrix
[CorrelationMatrix PvalueMatrix]=corrcoef(DataMatrix.');
CorrelationMatrixClass{1}=corrcoef(DataMatrix(:,Class{1}).');
CorrelationMatrixClass{2}=corrcoef(DataMatrix(:,Class{2}).');

% I want to save a Correlation matrix where we only have the points whose
% p-value is smaller than 1e-4 (significant), and changes those points into
% 0 if not
CorrelationMatrixP=CorrelationMatrix;
for i=1:size(PvalueMatrix,1)
    for j=1:size(PvalueMatrix,1)
        if i>j
            if PvalueMatrix(i,j)>=1e-4
                CorrelationMatrixP(i,j)=0;
                CorrelationMatrixP(j,i)=0;
            end
        end
    end
end

% I want to save a Correlation matrix where only correlations whose
% absolute value is >=0.3 are shown (like a mask of it).
CorrelationMatrixF=CorrelationMatrix;
for i=1:size(CorrelationMatrixF,1)
    for j=1:size(CorrelationMatrixF,1)
        if i>j
            if abs(CorrelationMatrixF(i,j))<0.3
                CorrelationMatrixF(i,j)=0;
                CorrelationMatrixF(j,i)=0;
            end
        end
    end
end
    

%% Now we want to show it as an image - Correlation Matrix

figure(1005)
colormap('gray')
imagesc(CorrelationMatrix,[-0.5 0.5])
colorbar
g=gca;
xlabel('Wavenumber / cm^{-1}')
ylabel('Wavenumber / cm^{-1}')
axis square

tickincrease=200;
set(gca, 'Xtick',1:tickincrease:length(XvaluesTrunc2))
set(gca, 'Xticklabel',round(XvaluesTrunc2(1:tickincrease:end)))
set(gca, 'XticklabelRotation',90)
set(gca, 'Ytick',1:tickincrease:length(XvaluesTrunc2))
set(gca, 'Yticklabel',XvaluesTrunc2(1:tickincrease:end))

% Lines for the places where the axis has a discontinuity
line([2503 2503],[0 length(XvaluesTrunc2)],'Color','w','Linewidth',2)
line([0 length(XvaluesTrunc2)],[2503 2503],'Color','w','Linewidth',2)
line([2815 2815],[0 length(XvaluesTrunc2)],'Color','w','Linewidth',2)
line([0 length(XvaluesTrunc2)],[2815 2815],'Color','w','Linewidth',2)

print(figure(1005),['CorrelationMatrix' '.tif'],'-dtiff','-r600')

%% Now we want to show it as an image - CorrelationMatrixP

figure(1006)
colormap('gray')
imagesc(CorrelationMatrixP,[-0.5 0.5])
colorbar
g=gca;
xlabel('Wavenumber / cm^{-1}')
ylabel('Wavenumber / cm^{-1}')
axis square

tickincrease=200;
set(gca, 'Xtick',1:tickincrease:length(XvaluesTrunc2))
set(gca, 'Xticklabel',round(XvaluesTrunc2(1:tickincrease:end)))
set(gca, 'XticklabelRotation',90)
set(gca, 'Ytick',1:tickincrease:length(XvaluesTrunc2))
set(gca, 'Yticklabel',XvaluesTrunc2(1:tickincrease:end))

% Lines for the places where the axis has a discontinuity
line([2503 2503],[0 length(XvaluesTrunc2)],'Color','w','Linewidth',2)
line([0 length(XvaluesTrunc2)],[2503 2503],'Color','w','Linewidth',2)
line([2815 2815],[0 length(XvaluesTrunc2)],'Color','w','Linewidth',2)
line([0 length(XvaluesTrunc2)],[2815 2815],'Color','w','Linewidth',2)

print(figure(1006),['CorrelationMatrixP' '.tif'],'-dtiff','-r600')

%% Now we want to show it as an image - CorrelationMatrixF

figure(1007)
colormap('gray')
imagesc(CorrelationMatrixF,[-0.3 0.3])
colorbar
g=gca;
xlabel('Wavenumber / cm^{-1}')
ylabel('Wavenumber / cm^{-1}')
axis square

tickincrease=200;
set(gca, 'Xtick',1:tickincrease:length(XvaluesTrunc2))
set(gca, 'Xticklabel',round(XvaluesTrunc2(1:tickincrease:end)))
set(gca, 'XticklabelRotation',90)
set(gca, 'Ytick',1:tickincrease:length(XvaluesTrunc2))
set(gca, 'Yticklabel',XvaluesTrunc2(1:tickincrease:end))

% Lines for the places where the axis has a discontinuity
line([2503 2503],[0 length(XvaluesTrunc2)],'Color','w','Linewidth',2)
line([0 length(XvaluesTrunc2)],[2503 2503],'Color','w','Linewidth',2)
line([2815 2815],[0 length(XvaluesTrunc2)],'Color','w','Linewidth',2)
line([0 length(XvaluesTrunc2)],[2815 2815],'Color','w','Linewidth',2)

print(figure(1007),['CorrelationMatrixF' '.tif'],'-dtiff','-r600')

%% Plot them as a draggable figure so we can see the correlation in each 
% point by clicking on it
Offset=0;
figure(400)

while 1==1
    subplot(2,1,1)
    AverageAll=mean(DataMatrix,2);
    plot(XvaluesTrunc2,AverageAll,'k')
    hold on
    xlim([XvaluesTrunc2(1) XvaluesTrunc2(end)])
    [j,y] = ginput(1);
    plot(j,AverageAll(find(round(j)==XvaluesTrunc2,1)),'ok')
    
    subplot(2,1,2)
    indextemp=find(round(j)==XvaluesTrunc2,1);
    plotCorr{count+1}=plot(XvaluesTrunc2,CorrelationMatrixP(:,indextemp)+...
        Offset*count);
    hold on
    line([XvaluesTrunc2(1) XvaluesTrunc2(end)],...
        [0.3+count*Offset 0.3+count*Offset],'Color','k')
    line([XvaluesTrunc2(1) XvaluesTrunc2(end)],...
        [-0.3+count*Offset -0.3+count*Offset],'Color','r')
    xlabel('Shift (cm^{-1})')
    ylabel('Correlation (Offset)')

    % Find peaks and label them
    [Peak, PeakIdx] = findpeaks(CorrelationMatrixP(:,indextemp),...
        'MinPeakProminence',0.1,'MinPeakHeight',0.1); 
    for i=1:length(Peak)
        text(XvaluesTrunc2(PeakIdx(i)), Peak(i)+Offset*count,...
            sprintf('%6.1f', XvaluesTrunc2(PeakIdx(i))));
    end    
    plot(XvaluesTrunc2(PeakIdx), Peak+Offset*count,'ok')

    legend(num2str(j))
    xlim([XvaluesTrunc2(1) XvaluesTrunc2(end)])
    hold off
end


%% PLRS Analysis - Partial Linear Squares Regression
% Now we want to do this analysis to see if there is any wavenumber in 
% particular that correlates with the disease stage: SW480<HT29<SW620
% For this we want to use the function plsregress

selectedClassesPLSR=[5,2,4];
count=0;
for i=selectedClassesPLSR
    if count==0
        IndexPLSR=Class{i};
        count=count+1;
    else
        IndexPLSR=[IndexPLSR Class{i}];
    end
end

% We need to create a Responses array that contains information about the
% disease stage, with 1 for SW480, 2 for HT29 and 3 for SW620
yPLRS=[ones(length(Class{2}),1);...
    2.*ones(length(Class{4}),1);...
    3.*ones(length(Class{5}),1)];

numcomps=2; %number of components, we can base it on the MSE or on the 95% rule
k=10; % cross validation k value
MCrepetitions=5; % monte-carlo repetitions

[XL,YL,XS,YS,BETA,PCTVAR,MSE,stats]=plsregress(DataMatrix(:,IndexPLSR).',...
    yPLRS,numcomps,'cv',k,'mcreps',MCrepetitions);

% Where each of the variables are:
% XL: the loadings of X
% YL: the loadings of Y, explain what the weight of each XL needs to be to
% explain the Y loadings given.
% XS: The value of each observation in the obtained components ie: like the
% scores in PCA
% YS:  response scores YS, that is, the linear combinations of the 
% responses with which the PLS components XS have maximum covariance. YS
% has a number of columns same as the number of components.
% BETA: it's like the ordinate and slope in the linear fit, so 
%           Y = [ones(n,1),X]*BETA + Yresiduals,
% BETA should be telling us how important each of the wavenumber are
% are.
% PCTVAR: variance explained by each component, in X (row 1) and Y (row 2)
% It compares the predicted values to the actual values obtained by
% the model in both X and Y.
% MSE: error for a model with 1:n components, can help identify how many
% components should be used
% stats: structured array that contains:
%   W  Weight of each component
%   T2  The T2 statistic for each point in XS.
%   Xresiduals  The predictor residuals, that is, X0-XS*XL'.
%   Yresiduals  The response residuals, that is, Y0-XS*YL'.

%% Plot some stats of it

figure
plot(1:numcomps,cumsum(100*PCTVAR(1,:)),'-bo');
hold on
plot(1:numcomps,cumsum(100*PCTVAR(2,:)),'-ro');
xlabel('Number of PLS components');
ylabel('Percent Variance Explained in y');
legend('Variance explained in X','Variance explained in Y')

figure
subplot(1,2,1)
plot(0:numcomps,MSE(1,:),'-bo');
xlabel('Number of PLS components');
ylabel('MSE for X')
subplot(1,2,2)
plot(0:numcomps,MSE(2,:),'-ko');
xlabel('Number of PLS components');
ylabel('MSE for Y')

figure % To see which cell line is fitting better
boxplot(stats.T2,yPLRS,'colors','k')


%% Plotting all the PLSR components shapes
OffsetPlot=0;
figure

selectedCoeffPLSR=1:numcomps;

count=0;
for j=selectedCoeffPLSR
    subplot('Position',[0.1,0.1,0.55,0.8])
    hold on
    plot(XvaluesTrunc2,XL(:,j)+OffsetPlot*count,'Color',colorsClass{j});
    %line([min(XvaluesTrunc2) 1750],[OffsetPlot*count OffsetPlot*count])
    subplot('Position',[0.7,0.1,0.2,0.8])
    hold on
    plot(XvaluesTrunc2,XL(:,j)+OffsetPlot*4*count,'Color',colorsClass{j});
    %line([2800 max(XvaluesTrunc2)],[OffsetPlot*count*4 OffsetPlot*count*4])
    count=count+1;
end

subplot('Position',[0.1,0.1,0.55,0.8])
xlim([min(XvaluesTrunc2) 1750])
ylim([-3 4])
%ylim([-(OffsetPlot+1) OffsetPlot*count])
xlabel('Wavenumber / cm^{-1}')
ylabel('PLSR loadings / Offset')
box on

subplot('Position',[0.7,0.1,0.2,0.8])
xlim([2800 max(XvaluesTrunc2)])
ylim([-12 16])
%ylim([-(OffsetPlot+1)*4 OffsetPlot*4*count])
box on
xlabel('Wavenumber / cm^{-1}')

legend('XL1','XL2')

%% Plot the box plots and beeswarms of the scores of PLRS

figure
hold on
count=1;
for k=1:2
    subplot(1,2,count)
    hold on
    BeeSwarmPlot=plotSpread(XS(:,k),'distributionIdx',yPLRS,...
        'categoryLabels',LegendStringClass(selectedClassesPLSR),...
        'distributionColors',colorsClass(selectedClassesPLSR),...
        'xNames',LegendStringClass(selectedClassesPLSR));
    boxplot(XS(:,k),yPLRS,'colors','k','symbol','')
    ylim([-0.2,0.2])
    ylabel(strcat('Score PLSR',num2str(k)))
    set(gca, 'XTickLabels', {'SW480','HT29','SW620'})
    set(gca, 'YTickLabels', {})
    set(gca, 'YTick', [])
    count=count+1;
end

%% t-Student Test on the X scores
% We want to see when the differences are significant between the groups so
% we know which components are interesting to look at. For that, we are
% running a 2-sample ttest where we compare each pair of cell lines, making
% sure we compare always the lower stage with the higher stage (SW480 with
% HT29, HT29 with SW620, SW480 with S620).

% It is important to note that we want the distribution to be left tailed
% so it is following the trend we are interested in (that is, that we want
% the first one to be significantly smaller than the next one, and not to
% be simply different).

for k=1:numcomps
    [tTest.h(k,1),tTest.p(k,1)] = ttest2(XS(yPLRS==1,k),...
        XS(yPLRS==2,k),'Alpha',0.001,'Tail','left');
    disp(strcat('p-value for component ', num2str(k),...
        ': SW480 vs HT29', ' is ', num2str(tTest.p(k,1))))
    
    [tTest.h(k,2),tTest.p(k,2)] = ttest2(XS(yPLRS==2,k),...
        XS(yPLRS==3,k),'Alpha',0.001,'Tail','left');
    disp(strcat('p-value for component ', num2str(k),...
        ': HT29 vs SW620', ' is ', num2str(tTest.p(k,2))))
    
    [tTest.h(k,3),tTest.p(k,3)] = ttest2(XS(yPLRS==1,k),...
        XS(yPLRS==3,k),'Alpha',0.001,'Tail','left');
    disp(strcat('p-value for component ', num2str(k),...
        ': SW480 vs SW620', ' is ', num2str(tTest.p(k,3))))
end
